@@ -1,17 +1,18 @@
 package Mouse::Meta::Class;
-use strict;
-use warnings;
+use Mouse::Util qw/:meta get_linear_isa not_supported/; # enables strict and warnings
 
 use Scalar::Util qw/blessed weaken/;
 
-use Mouse::Util qw/:meta get_linear_isa not_supported/;
-
 use Mouse::Meta::Method::Constructor;
 use Mouse::Meta::Method::Destructor;
 use Mouse::Meta::Module;
 our @ISA = qw(Mouse::Meta::Module);
 
-sub method_metaclass(){ 'Mouse::Meta::Method' } # required for get_method()
+sub method_metaclass()    { 'Mouse::Meta::Method'    }
+sub attribute_metaclass() { 'Mouse::Meta::Attribute' }
+
+sub constructor_class()   { 'Mouse::Meta::Method::Constructor' }
+sub destructor_class()    { 'Mouse::Meta::Method::Destructor'  }
 
 sub _construct_meta {
     my($class, %args) = @_;
@@ -25,10 +26,11 @@ sub _construct_meta {
         \@{ $args{package} . '::ISA' };
     };
 
-    #return Mouse::Meta::Class->initialize($class)->new_object(%args)
-    #    if $class ne __PACKAGE__;
-
-    return bless \%args, ref($class) || $class;
+    my $self = bless \%args, ref($class) || $class;
+    if(ref($self) ne __PACKAGE__){
+        $self->meta->_initialize_object($self, \%args);
+    }
+    return $self;
 }
 
 sub create_anon_class{
@@ -46,7 +48,13 @@ sub superclasses {
     my $self = shift;
 
     if (@_) {
-        Mouse::load_class($_) for @_;
+        foreach my $super(@_){
+            Mouse::Util::load_class($super);
+            my $meta = Mouse::Util::get_metaclass_by_name($super);
+            if($meta && $meta->isa('Mouse::Meta::Role')){
+                $self->throw_error("You cannot inherit from a Mouse Role ($super)");
+            }
+        }
         @{ $self->{superclasses} } = @_;
     }
 
@@ -57,6 +65,7 @@ sub find_method_by_name{
     my($self, $method_name) = @_;
     defined($method_name)
         or $self->throw_error('You must define a method name to find');
+
     foreach my $class( $self->linearized_isa ){
         my $method = $self->initialize($class)->get_method($method_name);
         return $method if defined $method;
@@ -77,6 +86,16 @@ sub get_all_method_names {
             $self->linearized_isa;
 }
 
+sub find_attribute_by_name{
+    my($self, $name) = @_;
+    my $attr;
+    foreach my $class($self->linearized_isa){
+        my $meta = Mouse::Util::get_metaclass_by_name($class) or next;
+        $attr = $meta->get_attribute($name) and last;
+    }
+    return $attr;
+}
+
 sub add_attribute {
     my $self = shift;
 
@@ -100,23 +119,16 @@ sub add_attribute {
             or $self->throw_error('You must provide a name for the attribute');
 
         if ($name =~ s/^\+//) { # inherited attributes
-            my $inherited_attr;
-
-            foreach my $class($self->linearized_isa){
-                my $meta = Mouse::Meta::Module::get_metaclass_by_name($class) or next;
-                $inherited_attr = $meta->get_attribute($name) and last;
-            }
-
-            defined($inherited_attr)
+            my $inherited_attr = $self->find_attribute_by_name($name)
                 or $self->throw_error("Could not find an attribute by the name of '$name' to inherit from in ".$self->name);
 
-            $attr = $inherited_attr->clone_and_inherit_options($name, \%args);
+            $attr = $inherited_attr->clone_and_inherit_options(%args);
         }
         else{
-            my($attribute_class, @traits) = Mouse::Meta::Attribute->interpolate_class($name, \%args);
+            my($attribute_class, @traits) = $self->attribute_metaclass->interpolate_class(\%args);
             $args{traits} = \@traits if @traits;
 
-            $attr = $attribute_class->new($name, \%args);
+            $attr = $attribute_class->new($name, %args);
         }
     }
 
@@ -131,16 +143,21 @@ sub add_attribute {
     return $attr;
 }
 
-sub compute_all_applicable_attributes { shift->get_all_attributes(@_) }
+sub compute_all_applicable_attributes {
+    Carp::cluck('compute_all_applicable_attributes() has been deprecated')
+        if _MOUSE_VERBOSE;
+    return shift->get_all_attributes(@_)
+}
+
 sub get_all_attributes {
     my $self = shift;
     my (@attr, %seen);
 
     for my $class ($self->linearized_isa) {
-        my $meta = $self->_metaclass_cache($class)
+        my $meta = Mouse::Util::get_metaclass_by_name($class)
             or next;
 
-        for my $name (keys %{ $meta->get_attribute_map }) {
+        for my $name ($meta->get_attribute_list) {
             next if $seen{$name}++;
             push @attr, $meta->get_attribute($name);
         }
@@ -155,14 +172,14 @@ sub new_object {
     my $self = shift;
     my %args = (@_ == 1 ? %{$_[0]} : @_);
 
-    my $instance = bless {}, $self->name;
+    my $object = bless {}, $self->name;
 
-    $self->_initialize_instance($instance, \%args);
-    return $instance;
+    $self->_initialize_object($object, \%args);
+    return $object;
 }
 
-sub _initialize_instance{
-    my($self, $instance, $args) = @_;
+sub _initialize_object{
+    my($self, $object, $args) = @_;
 
     my @triggers_queue;
 
@@ -171,18 +188,13 @@ sub _initialize_instance{
         my $key  = $attribute->name;
 
         if (defined($from) && exists($args->{$from})) {
-            $args->{$from} = $attribute->coerce_constraint($args->{$from})
-                if $attribute->should_coerce;
+            $object->{$key} = $attribute->_coerce_and_verify($args->{$from}, $object);
 
-            $attribute->verify_against_type_constraint($args->{$from});
-
-            $instance->{$key} = $args->{$from};
-
-            weaken($instance->{$key})
-                if ref($instance->{$key}) && $attribute->is_weak_ref;
+            weaken($object->{$key})
+                if ref($object->{$key}) && $attribute->is_weak_ref;
 
             if ($attribute->has_trigger) {
-                push @triggers_queue, [ $attribute->trigger, $args->{$from} ];
+                push @triggers_queue, [ $attribute->trigger, $object->{$key} ];
             }
         }
         else {
@@ -190,20 +202,14 @@ sub _initialize_instance{
                 unless ($attribute->is_lazy) {
                     my $default = $attribute->default;
                     my $builder = $attribute->builder;
-                    my $value = $attribute->has_builder
-                              ? $instance->$builder
-                              : ref($default) eq 'CODE'
-                                  ? $default->($instance)
-                                  : $default;
+                    my $value =   $builder                ? $object->$builder()
+                                : ref($default) eq 'CODE' ? $object->$default()
+                                :                           $default;
 
-                    $value = $attribute->coerce_constraint($value)
-                        if $attribute->should_coerce;
-                    $attribute->verify_against_type_constraint($value);
+                    $object->{$key} = $attribute->_coerce_and_verify($value, $object);
 
-                    $instance->{$key} = $value;
-
-                    weaken($instance->{$key})
-                        if ref($instance->{$key}) && $attribute->is_weak_ref;
+                    weaken($object->{$key})
+                        if ref($object->{$key}) && $attribute->is_weak_ref;
                 }
             }
             else {
@@ -216,35 +222,28 @@ sub _initialize_instance{
 
     foreach my $trigger_and_value(@triggers_queue){
         my($trigger, $value) = @{$trigger_and_value};
-        $trigger->($instance, $value);
+        $trigger->($object, $value);
     }
 
     if($self->is_anon_class){
-        $instance->{__METACLASS__} = $self;
+        $object->{__METACLASS__} = $self;
     }
 
-    return $instance;
+    return $object;
 }
 
 sub clone_object {
-    my $class    = shift;
-    my $instance = shift;
-    my %params   = (@_ == 1) ? %{$_[0]} : @_;
+    my $class  = shift;
+    my $object = shift;
+    my %params = (@_ == 1) ? %{$_[0]} : @_;
 
-    (blessed($instance) && $instance->isa($class->name))
-        || $class->throw_error("You must pass an instance of the metaclass (" . $class->name . "), not ($instance)");
+    (blessed($object) && $object->isa($class->name))
+        || $class->throw_error("You must pass an instance of the metaclass (" . $class->name . "), not ($object)");
 
-    my $clone = bless { %$instance }, ref $instance;
+    my $cloned = bless { %$object }, ref $object;
+    $class->_initialize_object($cloned, \%params);
 
-    foreach my $attr ($class->get_all_attributes()) {
-        if ( defined( my $init_arg = $attr->init_arg ) ) {
-            if (exists $params{$init_arg}) {
-                $clone->{ $attr->name } = $params{$init_arg};
-            }
-        }
-    }
-
-    return $clone;
+    return $cloned;
 }
 
 sub clone_instance {
@@ -260,17 +259,20 @@ sub make_immutable {
     my %args = (
         inline_constructor => 1,
         inline_destructor  => 1,
+        constructor_name   => 'new',
         @_,
     );
 
     $self->{is_immutable}++;
 
     if ($args{inline_constructor}) {
-        $self->add_method('new' => Mouse::Meta::Method::Constructor->generate_constructor_method_inline( $self ));
+        $self->add_method($args{constructor_name} =>
+            $self->constructor_class->_generate_constructor($self, \%args));
     }
 
     if ($args{inline_destructor}) {
-        $self->add_method('DESTROY' => Mouse::Meta::Method::Destructor->generate_destructor_method_inline( $self ));
+        $self->add_method(DESTROY =>
+            $self->destructor_class->_generate_destructor($self, \%args));
     }
 
     # Moose's make_immutable returns true allowing calling code to skip setting an explicit true value
@@ -284,7 +286,8 @@ sub is_immutable {  $_[0]->{is_immutable} }
 sub is_mutable   { !$_[0]->{is_immutable} }
 
 sub _install_modifier_pp{
-    my( $self, $into, $type, $name, $code ) = @_;
+    my( $self, $type, $name, $code ) = @_;
+    my $into = $self->name;
 
     my $original = $into->can($name)
         or $self->throw_error("The method '$name' is not found in the inheritance hierarchy for class $into");
@@ -349,7 +352,7 @@ sub _install_modifier_pp{
 }
 
 sub _install_modifier {
-    my ( $self, $into, $type, $name, $code ) = @_;
+    my ( $self, $type, $name, $code ) = @_;
 
     # load Class::Method::Modifiers first
     my $no_cmm_fast = do{
@@ -365,14 +368,14 @@ sub _install_modifier {
     else{
         my $install_modifier = Class::Method::Modifiers::Fast->can('_install_modifier');
         $impl = sub {
-            my ( $self, $into, $type, $name, $code ) = @_;
-            $install_modifier->(
-                $into,
-                $type,
-                $name,
-                $code
-            );
-            $self->{methods}{$name}++; # register it to the method map
+            my ( $self, $type, $name, $code ) = @_;
+            my $into = $self->name;
+            $install_modifier->($into, $type, $name, $code);
+
+            $self->add_method($name => do{
+                no strict 'refs';
+                \&{ $into . '::' . $name };
+            });
             return;
         };
     }
@@ -383,33 +386,64 @@ sub _install_modifier {
         *_install_modifier = $impl;
     }
 
-    $self->$impl( $into, $type, $name, $code );
+    $self->$impl( $type, $name, $code );
 }
 
 sub add_before_method_modifier {
     my ( $self, $name, $code ) = @_;
-    $self->_install_modifier( $self->name, 'before', $name, $code );
+    $self->_install_modifier( 'before', $name, $code );
 }
 
 sub add_around_method_modifier {
     my ( $self, $name, $code ) = @_;
-    $self->_install_modifier( $self->name, 'around', $name, $code );
+    $self->_install_modifier( 'around', $name, $code );
 }
 
 sub add_after_method_modifier {
     my ( $self, $name, $code ) = @_;
-    $self->_install_modifier( $self->name, 'after', $name, $code );
+    $self->_install_modifier( 'after', $name, $code );
 }
 
 sub add_override_method_modifier {
     my ($self, $name, $code) = @_;
 
+    if($self->has_method($name)){
+        $self->throw_error("Cannot add an override method if a local method is already present");
+    }
+
     my $package = $self->name;
 
-    my $body = $package->can($name)
+    my $super_body = $package->can($name)
         or $self->throw_error("You cannot override '$name' because it has no super method");
 
-    $self->add_method($name => sub { $code->($package, $body, @_) });
+    $self->add_method($name => sub {
+        local $Mouse::SUPER_PACKAGE = $package;
+        local $Mouse::SUPER_BODY    = $super_body;
+        local @Mouse::SUPER_ARGS    = @_;
+
+        $code->(@_);
+    });
+    return;
+}
+
+sub add_augment_method_modifier {
+    my ($self, $name, $code) = @_;
+    if($self->has_method($name)){
+        $self->throw_error("Cannot add an augment method if a local method is already present");
+    }
+
+    my $super = $self->find_method_by_name($name)
+        or $self->throw_error("You cannot augment '$name' because it has no super method");
+
+    my $super_package = $super->package_name;
+    my $super_body    = $super->body;
+
+    $self->add_method($name => sub{
+        local $Mouse::INNER_BODY{$super_package} = $code;
+        local $Mouse::INNER_ARGS{$super_package} = [@_];
+        $super_body->(@_);
+    });
+    return;
 }
 
 sub does_role {
@@ -419,8 +453,8 @@ sub does_role {
         || $self->throw_error("You must supply a role name to look for");
 
     for my $class ($self->linearized_isa) {
-        my $meta = Mouse::Meta::Module::class_of($class);
-        next unless $meta && $meta->can('roles');
+        my $meta = Mouse::Util::get_metaclass_by_name($class)
+            or next;
 
         for my $role (@{ $meta->roles }) {
 
@@ -432,18 +466,21 @@ sub does_role {
 }
 
 1;
-
 __END__
 
 =head1 NAME
 
 Mouse::Meta::Class - The Mouse class metaclass
 
+=head1 VERSION
+
+This document describes Mouse version 0.38
+
 =head1 METHODS
 
 =head2 C<< initialize(ClassName) -> Mouse::Meta::Class >>
 
-Finds or creates a Mouse::Meta::Class instance for the given ClassName. Only
+Finds or creates a C<Mouse::Meta::Class> instance for the given ClassName. Only
 one instance should exist for a given class.
 
 =head2 C<< name -> ClassName >>
@@ -454,30 +491,55 @@ Returns the name of the owner class.
 
 Gets (or sets) the list of superclasses of the owner class.
 
+=head2 C<< add_method(name => CodeRef) >>
+
+Adds a method to the owner class.
+
+=head2 C<< has_method(name) -> Bool >>
+
+Returns whether we have a method with the given name.
+
+=head2 C<< get_method(name) -> Mouse::Meta::Method | undef >>
+
+Returns a L<Mouse::Meta::Method> with the given name.
+
+Note that you can also use C<< $metaclass->name->can($name) >> for a method body.
+
+=head2 C<< get_method_list -> Names >>
+
+Returns a list of method names which are defined in the local class.
+If you want a list of all applicable methods for a class, use the
+C<get_all_methods> method.
+
+=head2 C<< get_all_methods -> (Mouse::Meta::Method) >>
+
+Return the list of all L<Mouse::Meta::Method> instances associated with
+the class and its superclasses.
+
 =head2 C<< add_attribute(name => spec | Mouse::Meta::Attribute) >>
 
 Begins keeping track of the existing L<Mouse::Meta::Attribute> for the owner
 class.
 
-=head2 C<< get_all_attributes -> (Mouse::Meta::Attribute) >>
-
-Returns the list of all L<Mouse::Meta::Attribute> instances associated with
-this class and its superclasses.
-
-=head2 C<< get_attribute_list -> { name => Mouse::Meta::Attribute } >>
-
-This returns a list of attribute names which are defined in the local
-class. If you want a list of all applicable attributes for a class,
-use the C<get_all_attributes> method.
-
 =head2 C<< has_attribute(Name) -> Bool >>
 
 Returns whether we have a L<Mouse::Meta::Attribute> with the given name.
 
-=head2 get_attribute Name -> Mouse::Meta::Attribute | undef
+=head2 C<< get_attribute Name -> Mouse::Meta::Attribute | undef >>
 
 Returns the L<Mouse::Meta::Attribute> with the given name.
 
+=head2 C<< get_attribute_list -> Names >>
+
+Returns a list of attribute names which are defined in the local
+class. If you want a list of all applicable attributes for a class,
+use the C<get_all_attributes> method.
+
+=head2 C<< get_all_attributes -> (Mouse::Meta::Attribute) >>
+
+Returns the list of all L<Mouse::Meta::Attribute> instances associated with
+this class and its superclasses.
+
 =head2 C<< linearized_isa -> [ClassNames] >>
 
 Returns the list of classes in method dispatch order, with duplicates removed.
@@ -488,12 +550,18 @@ Creates a new instance.
 
 =head2 C<< clone_object(Instance, Parameters) -> Instance >>
 
-Clones the given C<Instance> which must be an instance governed by this
+Clones the given instance which must be an instance governed by this
 metaclass.
 
+=head2 C<< throw_error(Message, Parameters) >>
+
+Throws an error with the given message.
+
 =head1 SEE ALSO
 
 L<Moose::Meta::Class>
 
+L<Class::MOP::Class>
+
 =cut
 
