@@ -1,8 +1,5 @@
 package Mouse::Meta::Method;
-use strict;
-use warnings;
-
-use Mouse::Util qw(:meta);
+use Mouse::Util qw(:meta); # enables strict and warnings
 
 use overload
     '&{}' => 'body',
@@ -32,8 +29,14 @@ __END__
 
 Mouse::Meta::Method - A Mouse Method metaclass
 
+=head1 VERSION
+
+This document describes Mouse version 0.38
+
 =head1 SEE ALSO
 
 L<Moose::Meta::Method>
 
+L<Class::MOP::Method>
+
 =cut
