@@ -1,5 +1,5 @@
 package Mouse::Meta::Role::Composite;
-use Mouse::Util; # enables strict and warnings
+use Mouse::Util qw(english_list); # enables strict and warnings
 use Mouse::Meta::Role;
 our @ISA = qw(Mouse::Meta::Role);
 
@@ -16,18 +16,16 @@ sub add_method {
         return;
     }
 
-    if($method_name eq 'meta'){
-        $self->SUPER::add_method($method_name => $code);
-    }
-    else{
-        # no need to add a subroutine to the stash
+    if($method_name ne 'meta'){
         my $roles = $self->{composed_roles_by_method}{$method_name} ||= [];
         push @{$roles}, $role;
         if(@{$roles} > 1){
             $self->{conflicting_methods}{$method_name}++;
         }
-        $self->{methods}{$method_name} = $code;
     }
+
+    $self->{methods}{$method_name} = $code;
+    # no need to add a subroutine to the stash
     return;
 }
 
@@ -52,9 +50,7 @@ sub has_override_method_modifier{
 }
 
 sub add_attribute{
-    my $self      = shift;
-    my $attr_name = shift;
-    my $spec      = (@_ == 1 ? $_[0] : {@_});
+    my($self, $attr_name, $spec) = @_;
 
     my $existing = $self->{attributes}{$attr_name};
     if($existing && $existing != $spec){
@@ -81,39 +77,38 @@ sub add_override_method_modifier{
 # components of apply()
 
 sub _apply_methods{
-    my($self, $consumer, $args) = @_;
+    my($self, $applicant, $args) = @_;
 
     if(exists $self->{conflicting_methods}){
-        my $consumer_class_name = $consumer->name;
+        my $applicant_class_name = $applicant->name;
 
-        my @conflicting = grep{ !$consumer_class_name->can($_) } keys %{ $self->{conflicting_methods} };
+        my @conflicting = sort grep{ !$applicant_class_name->can($_) } keys %{ $self->{conflicting_methods} };
 
         if(@conflicting == 1){
             my $method_name = $conflicting[0];
-            my $roles       = Mouse::Util::quoted_english_list(map{ $_->name } @{ $self->{composed_roles_by_method}{$method_name} });
+            my @roles       = sort @{ $self->{composed_roles_by_method}{$method_name} };
             $self->throw_error(
                sprintf q{Due to a method name conflict in roles %s, the method '%s' must be implemented or excluded by '%s'},
-                   $roles, $method_name, $consumer_class_name
+                   english_list(map{ sprintf q{'%s'}, $_->name } @roles), $method_name, $applicant->name
             );
         }
         elsif(@conflicting > 1){
+            my $methods = english_list(map{ sprintf q{'%s'}, $_ } @conflicting);
+
             my %seen;
-            my $roles = Mouse::Util::quoted_english_list(
-                grep{ !$seen{$_}++ } # uniq
-                map { $_->name }
-                map { @{$_} } @{ $self->{composed_roles_by_method} }{@conflicting}
+            my $roles   = english_list(
+                sort map{ my $name = $_->name; $seen{$name}++ ? () : sprintf q{'%s'}, $name }
+                map{ @{$_} } @{ $self->{composed_roles_by_method} }{@conflicting}
             );
 
             $self->throw_error(
                sprintf q{Due to method name conflicts in roles %s, the methods %s must be implemented or excluded by '%s'},
-                   $roles,
-                   Mouse::Util::quoted_english_list(@conflicting),
-                   $consumer_class_name
+                   $roles, $methods, $applicant->name
             );
         }
     }
 
-    $self->SUPER::_apply_methods($consumer, $args);
+    $self->SUPER::_apply_methods($applicant, $args);
     return;
 }
 1;
@@ -125,7 +120,7 @@ Mouse::Meta::Role::Composite - An object to represent the set of roles
 
 =head1 VERSION
 
-This document describes Mouse version 0.50_09
+This document describes Mouse version 0.38
 
 =head1 SEE ALSO
 
