@@ -1,8 +1,5 @@
 package Mouse::Object;
-use strict;
-use warnings;
-
-use Mouse::Util qw(does dump);
+use Mouse::Util qw(does dump); # enables strict and warnings
 
 sub new {
     my $class = shift;
@@ -33,7 +30,18 @@ sub BUILDARGS {
 sub DESTROY {
     my $self = shift;
 
-    $self->DEMOLISHALL();
+    local $?;
+
+    my $e = do{
+        local $@;
+        eval{
+            $self->DEMOLISHALL();
+        };
+        $@;
+    };
+
+    no warnings 'misc';
+    die $e if $e; # rethrow
 }
 
 sub BUILDALL {
@@ -77,25 +85,29 @@ __END__
 
 =head1 NAME
 
-Mouse::Object - we don't need to steenkin' constructor
+Mouse::Object - The base object for Mouse classes
+
+=head1 VERSION
+
+This document describes Mouse version 0.38
 
 =head1 METHODS
 
-=head2 C<< new arguments -> Object >>
+=head2 C<< new (Arguments) -> Object >>
 
 Instantiates a new C<Mouse::Object>. This is obviously intended for subclasses.
 
-=head2 C<< BUILDARGS(@args) -> HashRef >>
+=head2 C<< BUILDARGS (Arguments) -> HashRef >>
 
 Lets you override the arguments that C<new> takes. Return a hashref of
 parameters.
 
-=head2 C<< BUILDALL \%args >>
+=head2 C<< BUILDALL (\%args) >>
 
 Calls C<BUILD> on each class in the class hierarchy. This is called at the
 end of C<new>.
 
-=head2 C<< BUILD \%args >>
+=head2 C<< BUILD (\%args) >>
 
 You may put any business logic initialization in BUILD methods. You don't
 need to redispatch or return any specific value.
